<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://https://themeatelier.net/
 * @since      1.0.0
 *
 * @package    Postify_Pro
 * @subpackage Postify_Pro/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Postify_Pro
 * @subpackage Postify_Pro/admin
 * @author     ThemeAtelier <themeatelierbd@gmail.com>
 */
class Postify_Pro_Admin
{



	protected $suffix;

	protected $plugin_name;
	protected $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct($plugin_name, $version)
	{

		$this->suffix      = defined('SCRIPT_DEBUG') && SCRIPT_DEBUG || defined('WP_DEBUG') && WP_DEBUG ? '' : '.min';
		$this->plugin_name = $plugin_name;
		$this->version = $version;

		// Autoload system.
		spl_autoload_register(array($this, 'autoload'));

		POP_Metaboxes::layout_metabox('ta_pop_layouts');
		POP_Metaboxes::option_metabox('ta_pop_view_options');
		POP_Metaboxes::shortcode_metabox('ta_pop_display_shortcode');
		POP_Settings::settings('ta_postify_settings');

		$active_plugins = get_option('active_plugins');
		foreach ($active_plugins as $active_plugin) {
			$_temp = strpos($active_plugin, 'postify-pro.php');
			if (false != $_temp) {
				add_filter('plugin_action_links_' . $active_plugin, array($this, 'add_plugin_action_links'));
			}
		}
	}


	public function add_plugin_action_links($links)
	{
		$new_links = array(
			sprintf('<a href="%s">%s</a>', admin_url('post-new.php?post_type=postify'), esc_html__('Add New', 'postify-pro')),
			sprintf('<a href="%s">%s</a>', admin_url('edit.php?post_type=postify'), esc_html__('Settings', 'postify-pro')),
		);
		return array_merge($new_links, $links);
	}


	/**
	 * Autoload class files on demand
	 *
	 * @param string $class requested class name.
	 * @since 2.2.0
	 */
	private function autoload($class)
	{
		$name = explode('_', $class);
		if (isset($name[1])) {
			$class_name       = strtolower($name[1]);
			$pop_config_paths = array('views/', 'views/configs/settings/', 'views/configs/generator/');
			foreach ($pop_config_paths as $ta_pop_path) {
				$filename = plugin_dir_path(__FILE__) . '/' . $ta_pop_path . 'class-pop-' . $class_name . '.php';
				if (file_exists($filename)) {
					require_once $filename;
				}
			}
		}
	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Postify_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Postify_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		$current_screen        = get_current_screen();
		$the_current_post_type = $current_screen->post_type;
		if ('postify' == $the_current_post_type) {
			wp_enqueue_style('pop-font-awesome', POP_URL . 'public/assets/css/fontawesome.min.css', array(), POP_VERSION, 'all');
			wp_enqueue_style('ta-postify-admin', POP_URL . 'admin/assets/css/postify-pro-admin' . $this->suffix . '.css', array(), POP_VERSION, 'all');
		}
	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Postify_Pro_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Postify_Pro_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$current_screen        = get_current_screen();
		$the_current_post_type = $current_screen->post_type;
		if ('postify' == $the_current_post_type) {
			wp_enqueue_script('ta-postify-admin', POP_URL . 'admin/assets/js/postify-pro-admin' . $this->suffix . '.js', array('jquery'), POP_VERSION, true);
		}
	}

	/**
	 * Add postify admin columns.
	 *
	 * @since 2.0.0
	 * @return statement
	 */
	public function filter_postify_admin_column()
	{

		$admin_columns['cb']         = '<input type="checkbox" />';
		$admin_columns['title']      = esc_html__('Title', 'postify-pro');
		$admin_columns['shortcode']  = esc_html__('Shortcode', 'postify-pro');
		$admin_columns['pop_layout'] = esc_html__('Layout', 'postify-pro');
		$admin_columns['date']       = esc_html__('Date', 'postify-pro');

		return $admin_columns;
	}

	/**
	 * Display admin columns for the postifys.
	 *
	 * @param mix    $column The columns.
	 * @param string $post_id The post ID.
	 * @return void
	 */
	public function display_postify_admin_fields($column, $post_id)
	{

		$pop_layouts     = get_post_meta($post_id, 'ta_pop_layouts', true);
		$postifys_types = isset($pop_layouts['pop_layout_preset']) ? $pop_layouts['pop_layout_preset'] : '';
		switch ($column) {
			case 'shortcode':
				$column_field = '<input  class="ta_pop_input" style="width: 230px;padding: 4px 8px;cursor: pointer;" type="text" onClick="this.select();" readonly="readonly" value="[postify id=&quot;' . $post_id . '&quot;]"/> <div class="pop-after-copy-text"><i class="fa fa-check-circle"></i> Shortcode Copied to Clipboard! </div>';
				echo $column_field;
				break;
			case 'pop_layout':
				$layout = ucwords(str_replace('_layout', ' ', $postifys_types));
				esc_html_e($layout, 'postify-pro');
				break;
		} // end switch.
	}


	/**
	 * If it is the plugins page.
	 *
	 * @since 2.2.0
	 * @access private
	 */
	private function is_plugins_screen()
	{
		return in_array(get_current_screen()->id, ['plugins', 'plugins-network']);
	}
}

/**
 * Postify dashboard capability.
 *
 * @return string
 */
function pop_dashboard_capability()
{
	return apply_filters('pop_dashboard_capability', 'manage_options');
}
