<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://https://themeatelier.net/
 * @since      1.0.0
 *
 * @package    Postify_Pro
 * @subpackage Postify_Pro/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Postify_Pro
 * @subpackage Postify_Pro/includes
 * @author     ThemeAtelier <themeatelierbd@gmail.com>
 */
class Postify_Pro
{

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Postify_Pro_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name = POSTIFY_PRO_BASENAME;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version = POSTIFY_PRO_VERSION;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		$this->define_constants();
		$this->load_dependencies();
		$this->define_common_hooks();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->set_locale();
	}

	/**
	 * Define constant if not already set
	 *
	 * @since 2.2.0
	 *
	 * @param string      $name Define constant.
	 * @param string|bool $value Define constant.
	 */
	public function define($name, $value)
	{
		if (!defined($name)) {
			define($name, $value);
		}
	}

	/**
	 * Define constants
	 *
	 * @since 2.2.0
	 */
	public function define_constants()
	{
		$this->define('POP_VERSION', $this->version);
		$this->define('POP_PLUGIN_NAME', $this->plugin_name);
		$this->define('POP_PATH', plugin_dir_path(dirname(__FILE__)));
		$this->define('POP_TEMPLATE_PATH', plugin_dir_path(dirname(__FILE__)) . 'public/templates/');
		$this->define('POP_URL', plugin_dir_url(dirname(__FILE__)));
		$this->define('POSTIFY_PRO_BASENAME', POSTIFY_PRO_BASENAME);
	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Postify_Pro_Loader. Orchestrates the hooks of the plugin.
	 * - Postify_Pro_i18n. Defines internationalization functionality.
	 * - Postify_Pro_Admin. Defines all hooks for the admin area.
	 * - Postify_Pro_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once POP_PATH . 'includes/class-postify-pro-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once POP_PATH . 'includes/class-postify-pro-i18n.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once POP_PATH . 'includes/class-postify-pro-post-types.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */

		require_once POP_PATH . 'admin/class-postify-pro-admin.php';
		require_once POP_PATH . 'admin/views/ta-framework/classes/setup.class.php';


		/**
		 * The class responsible for defining metabox config that occur in the admin area.
		 */
		require_once POP_PATH . 'admin/helpers/class-postify-pro-image-resizer.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once POP_PATH . 'public/class-postify-pro-public.php';

		$this->loader = new Postify_Pro_Loader();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Postify_Pro_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{

		$plugin_i18n = new Postify_Pro_i18n();

		$this->loader->add_action('plugins_loaded', $plugin_i18n, 'load_plugin_textdomain');
	}

	/**
	 * Register common hooks.
	 *
	 * @since 1.0.0
	 * @access private
	 */
	private function define_common_hooks()
	{
		$common_hooks = new Postify_pro_Post_Type(POP_PLUGIN_NAME, POP_VERSION);
		$this->loader->add_action('init', $common_hooks, 'register_postify_post_type', 10);
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{
		$plugin_admin = new Postify_Pro_Admin(POP_PLUGIN_NAME, POP_VERSION);


		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_styles');
		$this->loader->add_action('admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts');

		$this->loader->add_filter('manage_postify_posts_columns', $plugin_admin, 'filter_postify_admin_column');
		$this->loader->add_action('manage_postify_posts_custom_column', $plugin_admin, 'display_postify_admin_fields', 10, 2);
	}



	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{

		$plugin_public = new Postify_Pro_Public(POP_PLUGIN_NAME, POP_VERSION);

		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_styles');
		$this->loader->add_action('wp_enqueue_scripts', $plugin_public, 'enqueue_scripts');
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Postify_Pro_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}
}
